@@ -74,10 +74,22 @@ class LCRandom31(RandomBase):
 class MTRandom(RandomBase, random2.Random):
     MAX_SEED = 100000000000000000000  # 20 digits
 
+    def __init__(self, seed=None):
+        if seed is None:
+            seed = self._getRandomSeed()
+        RandomBase.__init__(self)
+        random2.Random.__init__(self, seed)
+        self.initial_seed = seed
+        self.initial_state = self.getstate()
+        self.origin = self.ORIGIN_UNKNOWN
+
     def setSeed(self, seed):
         random2.Random.__init__(self, seed)
         self.initial_state = self.getstate()
 
+    def reset(self):
+        self.setstate(self.initial_state)
+
 
 def shuffle(cards, game_num, which_deals):
     ms = ((game_num <= 32000) or (which_deals == RandomBase.DEALS_MS))
@@ -86,7 +98,10 @@ def shuffle(cards, game_num, which_deals):
     r.setSeed(game_num)
     return r.shuffle(ms_rearrange(cards) if ms else cards)
 
+
 _ms_pat = re.compile("ms([0-9]+)\\n?\\Z")
+
+
 def match_ms_deal_prefix(mystring):
     ret = re.match(_ms_pat, mystring)
     return None if not ret else int(ret.group(1))
